<?php

require_once 'lib/i18n.php';
require_once 'lib/user_input_handler.php';
require_once '24tv.php';

define('ERR_INIT_LIB',          101);
define('ERR_INIT_DATA',         102);

if (!function_exists('curl_init'))    throw new Exception('TvigleRest needs the CURL PHP extension.', ERR_INIT_LIB);
if (!function_exists('json_decode'))  throw new Exception('TvigleRest  needs the JSON PHP extension.', ERR_INIT_LIB);

class _24tvApi {

    // URL to the online stream
    public static $favorites_url = 'favorites';

    // video items view params
    public static $movie_list_vip = array (
        ViewItemParams::item_paint_icon => true,
        ViewItemParams::item_layout => HALIGN_LEFT,
        ViewItemParams::icon_valign => VALIGN_CENTER,
        ViewItemParams::icon_dx => 20,
        ViewItemParams::icon_dy => -5,
        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
        ViewItemParams::item_caption_width => 1100
    );

    // ID of the "Online" group
    public static $online_group_id = 0;

    // ID of the "tv24" channel
    public static $tv24_channel_id = 0;

    // ID of the "Categories" group
    public static $categories_group_id = 1;

    // URL to the online stream
    public static $online_url = 'http://ts://stream1.luxnet.ua/news24/smil:news24.streamsamsung.smil/playlist.m3u8?secretToken=DuneTVbddfggneret58gfjDg65jjynuyu';

    // URL to the RSS feed
    // todo: replace with dune RSS feed
    private static $RSSUrl = 'http://smarttv.24tv.ua/dune/resources/xml/smartTvCategory.xml'; // <<< PRODUCTION
    // private static $RSSUrl = 'http://192.168.2.6:8080/dune/resources/xml/smartTvCategory.xml';

    // URL to the RSS feed category, _ID_ should be replaced with actual id got from the main RSS
    // todo: replace with dune RSS feed
    private static $RSSCategoryUrl = 'http://smarttv.24tv.ua/dune/resources/xml/smartTvNews_ID_.xml'; // <<< PRODUCTION
    // private static $RSSCategoryUrl = 'http://192.168.2.6:8080/dune/resources/xml/smartTvNews_ID_.xml';

    // string constant to be replaced in RSSCategoryUrl
    private static $idStrConst = '_ID_';

    // URL to the online group icon
    private static $online_group_icon_url = 'plugin_file://icons/icon-online.png';
    
    private static $online_group_icon94_url = 'plugin_file://icons/online-94x94.png';
    
    // URL to the favorites icon for main menu
    public static $favorites_icon_url = 'plugin_file://icons/icon-favorites.png';

    // URL to the online icon for main menu
    private static $tv24_channel_icon_url = 'plugin_file://icons/online-300x225.png';

    // URL to the categories group icon
    private static $categories_group_icon_url = 'plugin_file://icons/category-94x94.png';
    
    public static $fav_separator = "###";
    
    // days back that could be scrolled in daily EPG box
    private static $maxDaysBack = 10000;
    
    // groups
    private $groups = null;

    // channels
    private $channels = null;

    // root menu
    private $rootMenuFolders = null;

    // categoryNews array
    private $categoryNews = array();

    // associates news itme timestamp to it index for currently selected date
    // date is selected in 
    public $dailyEPGTimestampToIndex = array();
    
    public static $piwikSiteId = 20;
    public static $piwikPlayVideoIdGoal = 1;
    public static $piwikPlayOnlineIdGoal = 2;
    public static $piwikDeviceId = 777;

    // curl options
    private static $CURL_OPTS = array (
        CURLOPT_CONNECTTIMEOUT => 10,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT        => 60,
        CURLOPT_USERAGENT      => 'dune'
    );

    public function __construct() {
        //hd_print("======= 24tvAPI __construct");
        $vip = self::$movie_list_vip;
        // enclosure holds thumb image
        $vip[ViewItemParams::item_detailed_icon_path] = self::$tv24_channel_icon_url;
        $vip[ViewItemParams::icon_path] = 'gui_skin://small_icons/movie.aai';
        $vip[ViewItemParams::item_detailed_info] = i18n::getText('newsView.online') . ' | ';
        $this->categoryNews[self::$tv24_channel_id][] = 
            array (
                // timestamp is already encoded in media_url, duplicating to quickly access in get_day_epg
                'timestamp' => 0,
                PluginRegularFolderItem::caption => i18n::getText('channels.tv24'),
                PluginRegularFolderItem::view_item_params => $vip,
                PluginRegularFolderItem::media_url => 
                    HD::encode_user_data(
                        self::$online_url,
                        array ('channel_id' => self::$tv24_channel_id, 'news_item_index' => 0,
                            'timestamp' => 0))
            );
        //hd_print("======= 24tvAPI __construct end");
    }

    public function getGroups() {
        //hd_print("======= getGroups");
        if($this->groups == null) {
            // 2 groups: Favorites & Categories
            $this->groups = array();
            array_push($this->groups,
                array (
                    PluginTvGroup::id => self::$online_group_id,
                    PluginTvGroup::caption => i18n::getText('groups.online'),
                    PluginTvGroup::icon_url => self::$online_group_icon94_url
                ),
                array (
                    PluginTvGroup::id => self::$categories_group_id,
                    PluginTvGroup::caption => i18n::getText('groups.categories'),
                    PluginTvGroup::icon_url => self::$categories_group_icon_url
                ));
        }
        return $this->groups;
    }

    public function getRootMenuFolders() {
        //hd_print("======= getRootMenuFolders");
        if($this->rootMenuFolders == null) {
            // permanent menus
            $this->rootMenuFolders = array();
/*
            $this->rootMenuFolders = array (
                array (
                    PluginRegularFolderItem::caption => i18n::getText('menu.favorites'),
                    PluginRegularFolderItem::view_item_params => array (ViewItemParams::icon_path => self::$favorites_icon_url),
                    PluginRegularFolderItem::media_url => self::$favorites_url
                )
            );
*/
            $channels = $this->getChannels();
            foreach($channels as $channel) {
                // skip 24 online
                //if ($channel[PluginTvChannel::id] == self::$tv24_channel_id) continue;

                $this->rootMenuFolders[] = array (
                    PluginRegularFolderItem::caption => $channel[PluginTvChannel::caption],
                    PluginRegularFolderItem::view_item_params =>
                        array (ViewItemParams::icon_path => $channel[PluginTvChannel::icon_url]),
                    PluginRegularFolderItem::media_url =>   $channel[PluginTvChannel::id]
                );
            }
        }
        return $this->rootMenuFolders;
    }

    public function getChannels() {
        //hd_print("======= getChannels");
        if($this->channels == null) {
            $this->channels = array();
            
            // online channel
            $this->channels[] =
                array (
                    PluginTvChannel::id => self::$tv24_channel_id,
                    PluginTvChannel::number => 1,
                    PluginTvChannel::caption => i18n::getText('channels.tv24'),
                    PluginTvChannel::group_ids => array(self::$online_group_id),
                    PluginTvChannel::icon_url => self::$online_group_icon_url,
                    PluginTvChannel::recording_enabled => true,
                    PluginTvChannel::playback_url_is_stream_url => true
                );
            
            $this->channels[] =
                array (
                        PluginTvChannel::id => self::$favorites_url,
                        PluginTvChannel::number => 1,
                        PluginTvChannel::caption => i18n::getText('channels.favorites'),
                        PluginTvChannel::group_ids => array(self::$categories_group_id),
                        PluginTvChannel::icon_url => strval($this->get_favorites_icon_url()),
                    
                        PluginTvChannel::have_archive => true,
                        PluginTvChannel::recording_enabled => true,

                        PluginTvChannel::past_epg_days => self::$maxDaysBack,
                        PluginTvChannel::future_epg_days => 0,

                        PluginTvChannel::archive_past_sec => PHP_INT_MAX,
                        PluginTvChannel::archive_delay_sec => 0,

                        PluginTvChannel::playback_url_is_stream_url => true
                );
            
            $categories = $this->fetchData(self::$RSSUrl);
             foreach($categories as $category) {
//-------------
                $icon = $category['thumbImage'];
                if ($category['id'] == 2) //  
                     $icon ='plugin_file://icons/icon-ukraine.png';
                else if ($category['id'] == 3) // 
                     $icon ='plugin_file://icons/icon-programs.png';
                else if ($category['id'] == 4) // 
                     $icon ='plugin_file://icons/icon-sport.png';
                else if ($category['id'] == 5) // 
                     $icon ='plugin_file://icons/all.png';
                else if ($category['id'] == 6) // 
                     $icon ='plugin_file://icons/icon-eco.png';
//------------------------
                $this->channels[] =
                    array (
                        PluginTvChannel::id => $category['link'],
                        PluginTvChannel::number => $category['id'],
                        PluginTvChannel::caption => $category['title'],
                        PluginTvChannel::group_ids => array(self::$categories_group_id),
                        PluginTvChannel::icon_url => $icon, //-----------------

                        PluginTvChannel::have_archive => true,
                        PluginTvChannel::recording_enabled => true,

                        PluginTvChannel::past_epg_days => self::$maxDaysBack,
                        PluginTvChannel::future_epg_days => 0,

                        PluginTvChannel::archive_past_sec => PHP_INT_MAX,
                        PluginTvChannel::archive_delay_sec => 0,

                        PluginTvChannel::playback_url_is_stream_url => true
                    );
            }
        }
        return $this->channels;
    }

    ///////////////////////////////////////////////////////////////////////
    //
    public function getCategoryNews($media_url) {
        //hd_print("======= getCategoryNews, media_url=" . $media_url);

        $media_str = null; $user_data = null;
        HD::decode_user_data($media_url, $media_str, $user_data);
        if(!array_key_exists($media_str, $this->categoryNews)) {
            //hd_print("======= getCategoryNews, INIT for=" . $media_str);
            $categoryUrl = str_replace(self::$idStrConst, $media_str, self::$RSSCategoryUrl);
            $news = $this->fetchData($categoryUrl);
            $this->categoryNews[$media_str] = array();
            $this->dailyEPGTimestampToIndex [$media_str] = array();

            $news_item_index = 0;
            $timestampsArr = array();
            foreach($news as $newsItem)  {
                // TODO: !!!
                // pubDate in RSS should be in rfc2822 format: http://www.php.net/manual/en/class.datetime.php#datetime.constants.rfc2822
                // code below is temporal fix
                // =========== start: fix RSS and REMOVE this 
                $ukrTime = null;
                if(strlen($newsItem['pubDate']) == 5) { // if pubDate is time only value like: 17:39
                    $ukrTime = new DateTime($newsItem['pubDate'], new DateTimeZone('Europe/Kiev'));
                } else {
                    $ukrTime = DateTime::createFromFormat('H:i d.m.y', $newsItem['pubDate'], new DateTimeZone('Europe/Kiev'));
                }
                // =========== end: fix RSS and REMOVE this
                // should be something like:
                // $ukrTime = new DateTime($newsItem['pubDate']);
                
                // each news should have unique timestamp
                // add 1 second if there are news with the same pubdate
                $timestamp = $ukrTime->getTimestamp();
                while (array_key_exists($timestamp, $timestampsArr)) {
                    //hd_print("======= getCategoryNews, add 1 second to timestamp=$timestamp");
                    $timestamp++;
                }
                $timestampsArr[$timestamp] = $timestamp;
                
                $vip = self::$movie_list_vip;
                // enclosure holds thumb image
                $vip[ViewItemParams::item_detailed_icon_path] = $newsItem['thumbImage'];
                $vip[ViewItemParams::icon_path] = 'gui_skin://small_icons/movie.aai';
                $vip[ViewItemParams::item_detailed_info] = i18n::getText('newsView.pubDate') . ' | ' . HD::toLocalFormattedDateTime($timestamp) .
                        '||' . i18n::getText('newsView.description') . ' | ' . $newsItem['description'];
                $this->categoryNews[$media_str][] =
                    array (
                        // timestamp is already encoded in media_url, duplicating to quickly access in get_day_epg
                        'timestamp' => $timestamp,
                        PluginRegularFolderItem::caption => $newsItem['title'],
                        // PluginRegularFolderItem::description => $newsItem['description'],
                        // description
                        PluginRegularFolderItem::view_item_params => $vip,
                        PluginRegularFolderItem::media_url => 
                            HD::encode_user_data(
                                'http://mp4://' . substr($newsItem['link'], 7),
                                array ('channel_id' => $media_str, 
                                       'news_item_index' => $news_item_index,
                                       'timestamp' => $timestamp,
                                       'caption' => str_replace("\"","'",$newsItem['title']),
                                       'thumbImage' => $newsItem['thumbImage']))
                    );
                $this->dailyEPGTimestampToIndex[$media_str][$timestamp] = $news_item_index;
                //hd_print("======= getCategoryNews, PUT=$media_str/$timestamp");
                $news_item_index++;
            }
        }
        return $this->categoryNews[$media_str];
    }

    ///////////////////////////////////////////////////////////////////////
    // getIndexByTimestamp
    public function getIndexByTimestamp($channel_id, $archive_ts) {
        if(array_key_exists($archive_ts, $this->dailyEPGTimestampToIndex[$channel_id])) {
            return $this->dailyEPGTimestampToIndex[$channel_id][$archive_ts];
        } else {
            return -1;
        }
    }
    
    public function is_favorite_channel_id($channel_id, &$plugin_cookies) {
        if (!$this->is_favorites_supported())
            return false;

        $fav_channel_ids = $this->get_fav_channel_ids($plugin_cookies);

        $k = array_search($channel_id, $fav_channel_ids);
        return $k !== false;
    }
    
    public function change_tv_favorites($fav_op_type, $channel_id, &$plugin_cookies) {
        
        $fav_channel_ids = $this->get_fav_channel_ids($plugin_cookies);
        
        if ($fav_op_type === PLUGIN_FAVORITES_OP_ADD) {
            array_push($fav_channel_ids, $channel_id);
        } else if ($fav_op_type === PLUGIN_FAVORITES_OP_REMOVE) {

            $k = array_search($channel_id, $fav_channel_ids);
            if ($k !== false) {
                unset ($fav_channel_ids[$k]);
            }
        }
        $this->set_fav_channel_ids($plugin_cookies, $fav_channel_ids);
    }
    
    public function is_favorites_supported() { return true; }
    
    public function get_favorites_icon_url() { return self::$favorites_icon_url; }
    
    public function get_fav_channel_ids(&$plugin_cookies) {
        $fav_channel_ids = array();

        if (isset($plugin_cookies->{'favorite_channels'}))
            $fav_channel_ids = preg_split("/".self::$fav_separator."/", $plugin_cookies->{'favorite_channels'});

        return $fav_channel_ids;
    }
    
    public function set_fav_channel_ids(&$plugin_cookies, &$ids) {
        $plugin_cookies->{'favorite_channels'} = join(self::$fav_separator, $ids);
        //$plugin_cookies->{'favorite_channels'} = "";
    }
    
    private function fetchData($url) {
        //hd_print("======= fetchData, url=" . $url);
        try {
            $this->sLastRes = $this->request($url);
            $res = $this->parseRSSToItemsArr($this->sLastRes);
        } catch (Exception $e) {
            //hd_print("Dune webapp request failed for url: " . $url);
            $res = array();
        }
        
        return $res;
    }

    private function parseRSSToItemsArr($data) {
        //hd_print("======= parseRSSToItemsArr");
        $xml = new SimpleXmlElement($data, LIBXML_NOCDATA);
        $cnt = count($xml->channel->item);
        $arXML = array ();
        for($i=0; $i<$cnt; $i++) {
            $arXML[$i]=array ();
            $arXML[$i]['link'] = (string)$xml->channel->item[$i]->link;
            $arXML[$i]['title'] = (string)$xml->channel->item[$i]->title;
            if ($xml->channel->item[$i]->description != null) {
                $arXML[$i]['description'] = (string)$xml->channel->item[$i]->description;
            }
            if ($xml->channel->item[$i]->id != null) {
                $arXML[$i]['id'] = (string)$xml->channel->item[$i]->id;
            }
            if ($xml->channel->item[$i]->enclosure != null) {
                // grab the first enclosure
                $enclosure = $xml->channel->item[$i]->enclosure[0]->attributes();
                $arXML[$i]['thumbImage'] = (string)$enclosure['url'];
            }
            if ($xml->channel->item[$i]->pubDate != null) {
                $arXML[$i]['pubDate'] = (string)$xml->channel->item[$i]->pubDate;
            }
        }
        return $arXML;
    }

    public function request($url) {
        //hd_print("======= request--> $url");
        $ch = curl_init();
        $opts = self::$CURL_OPTS;
        $opts[CURLOPT_URL] = $url;
        curl_setopt_array ($ch, $opts);
        $result = curl_exec($ch);
        $nCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($result === false) {
            $err_msg = "CURL error: $nCode (" . curl_error($ch) . ')';
            //hd_print($err_msg);
            throw new Exception($err_msg);
        }
        if($nCode != 200) {
            $err_msg = "API request failed ($nCode)";
            //hd_print($err_msg);
            //hd_print("URL: $url");
            throw new Exception($err_msg);
        }
        curl_close($ch);
        return $result;
    }

}
?>
